#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IBaseParagraph, IParagraphFactory, IParagraphContainerTarget, \
    IParagraphContainer, IParagraphFactorySettings
from pyams_content.features.preview.interfaces import IPreviewTarget
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_form.interfaces.form import IFormContextPermissionChecker
from pyams_i18n.interfaces import II18n
from pyams_workflow.interfaces import IWorkflowState
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent

# import packages
from persistent import Persistent
from pyams_content.features.checker import BaseContentChecker
from pyams_content.features.renderer import RenderedContentMixin
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.registry import query_utility
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from pyramid.events import subscriber
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import ObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


#
# Auto-creation of default paragraphs
#

@subscriber(IObjectAddedEvent, context_selector=IParagraphContainerTarget)
def handle_new_paragraphs_container(event):
    """Handle new paragraphs container

    Subscriber to IObjectAddedEvent for objects implementing IParagraphContainerTarget: when a
    new paragraphs container is created (in version 1), automatically create new paragraphs matching

    """
    container = IParagraphContainer(event.object)
    if len(container) > 0:
        return
    content = get_parent(container, IWfSharedContent)
    version_state = IWorkflowState(content, None) if content is not None else None
    if (version_state is None) or (version_state.version_id == 1):
        # only apply to first version
        settings = get_parent(container, IParagraphFactorySettings)
        if settings is not None:
            for factory_name in settings.auto_created_paragraphs or ():
                factory = query_utility(IParagraphFactory, name=factory_name)
                if factory is not None:
                    container.append(factory.content_type())


#
# Base paragraph classes and subscribers
#

@implementer(IBaseParagraph, IPreviewTarget)
class BaseParagraph(RenderedContentMixin, Persistent, Contained):
    """Base paragraph persistent class"""

    icon_class = ''
    icon_hint = ''

    visible = FieldProperty(IBaseParagraph['visible'])
    title = FieldProperty(IBaseParagraph['title'])

    empty_title = ' -' * 8


@implementer(IParagraphFactory)
class BaseParagraphFactory(object):
    """Base paragraph factory class"""

    name = None
    content_type = None
    secondary_menu = False


class BaseParagraphContentChecker(BaseContentChecker):
    """Base paragraph content checker mixin"""

    @property
    def label(self):
        request = check_request()
        translate = request.localizer.translate
        return II18n(self.context).query_attribute('title', request) or \
            '({0})'.format(translate(self.context.icon_hint).lower())


@vocabulary_config(name='PyAMS paragraph factories')
class ParagraphFactoriesVocabulary(SimpleVocabulary):
    """Paragraph factories vocabulary"""

    def __init__(self, context=None):
        request = check_request()
        registry = request.registry
        translate = request.localizer.translate
        terms = sorted([SimpleTerm(name, title=translate(util.name))
                        for name, util in registry.getUtilitiesFor(IParagraphFactory)],
                       key=lambda x: x.title)
        super(ParagraphFactoriesVocabulary, self).__init__(terms)


@vocabulary_config(name='PyAMS content paragraphs')
class ContentParagraphsVocabulary(SimpleVocabulary):
    """Content paragraphs vocabulary"""

    def __init__(self, context):
        request = check_request()
        if not IParagraphContainerTarget.providedBy(context):
            context = get_parent(context, IParagraphContainerTarget)
        if context is not None:
            terms = [SimpleTerm(para.__name__,
                                title='§{0}: {1}'.format(index+1,
                                                         II18n(para).query_attribute('title', request=request) or
                                                         BaseParagraph.empty_title))
                     for index, para in enumerate(IParagraphContainer(context).values())]
        else:
            terms = []
        super(ContentParagraphsVocabulary, self).__init__(terms)


@adapter_config(context=IBaseParagraph, provides=IFormContextPermissionChecker)
class BaseParagraphPermissionChecker(ContextAdapter):
    """Paragraph permission checker"""

    @property
    def edit_permission(self):
        content = get_parent(self.context, IParagraphContainerTarget)
        return IFormContextPermissionChecker(content).edit_permission


@subscriber(IObjectAddedEvent, context_selector=IBaseParagraph)
def handle_added_paragraph(event):
    """Handle added paragraph"""
    content = get_parent(event.object, IParagraphContainerTarget)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectModifiedEvent, context_selector=IBaseParagraph)
def handle_modified_paragraph(event):
    """Handle modified paragraph"""
    content = get_parent(event.object, IParagraphContainerTarget)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectRemovedEvent, context_selector=IBaseParagraph)
def handle_removed_paragraph(event):
    """Handle removed paragraph"""
    content = get_parent(event.object, IParagraphContainerTarget)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))
